@@ -47,6 +47,7 @@ import org.mockito.stubbing.Answer;
 
 import static org.hamcrest.CoreMatchers.not;
 import static org.junit.Assert.assertThat;
+import static org.mockito.ArgumentMatchers.any;
 
 public class TestAuthenticationFilter {
 
@@ -561,7 +562,7 @@ public class TestAuthenticationFilter {
             return null;
           }
         }
-      ).when(chain).doFilter(Mockito.<ServletRequest>anyObject(), Mockito.<ServletResponse>anyObject());
+      ).when(chain).doFilter(any(), any());
 
       Mockito.when(response.containsHeader("WWW-Authenticate")).thenReturn(true);
       filter.doFilter(request, response, chain);
@@ -644,8 +645,8 @@ public class TestAuthenticationFilter {
         Assert.assertNotNull("cookie missing", v);
         Assert.assertTrue(v.contains("u=") && v.contains("p=") && v.contains
                 ("t=") && v.contains("e=") && v.contains("s="));
-        Mockito.verify(chain).doFilter(Mockito.any(ServletRequest.class),
-                Mockito.any(ServletResponse.class));
+        Mockito.verify(chain).doFilter(any(ServletRequest.class),
+                any(ServletResponse.class));
 
         SignerSecretProvider secretProvider =
             StringSignerSecretProviderCreator.newStringSignerSecretProvider();
@@ -755,7 +756,7 @@ public class TestAuthenticationFilter {
             return null;
           }
         }
-      ).when(chain).doFilter(Mockito.<ServletRequest>anyObject(), Mockito.<ServletResponse>anyObject());
+      ).when(chain).doFilter(any(), any());
 
       filter.doFilter(request, response, chain);
 
@@ -808,7 +809,7 @@ public class TestAuthenticationFilter {
             return null;
           }
         }
-      ).when(chain).doFilter(Mockito.<ServletRequest>anyObject(), Mockito.<ServletResponse>anyObject());
+      ).when(chain).doFilter(any(), any());
 
       filter.doFilter(request, response, chain);
 
@@ -1118,8 +1119,8 @@ public class TestAuthenticationFilter {
       Assert.assertTrue(v.contains("u=") && v.contains("p=") && v.contains
           ("t=") && v.contains("i=") && v.contains("e=")
           && v.contains("s="));
-      Mockito.verify(chain).doFilter(Mockito.any(ServletRequest.class),
-          Mockito.any(ServletResponse.class));
+      Mockito.verify(chain).doFilter(any(ServletRequest.class),
+          any(ServletResponse.class));
 
       SignerSecretProvider secretProvider =
           StringSignerSecretProviderCreator.newStringSignerSecretProvider();
@@ -1162,8 +1163,8 @@ public class TestAuthenticationFilter {
 
     Mockito.verify(response).sendError(Mockito.eq(HttpServletResponse
             .SC_UNAUTHORIZED), Mockito.anyString());
-    Mockito.verify(chain, Mockito.never()).doFilter(Mockito.any
-            (ServletRequest.class), Mockito.any(ServletResponse.class));
+    Mockito.verify(chain, Mockito.never()).doFilter(
+        any(ServletRequest.class), any(ServletResponse.class));
 
     Assert.assertTrue("cookie is missing",
         cookieMap.containsKey(AuthenticatedURL.AUTH_COOKIE));
