@@ -19,7 +19,7 @@ package org.apache.hadoop.oncrpc.security;
 
 import org.apache.hadoop.oncrpc.XDR;
 
-import com.google.common.base.Preconditions;
+import org.apache.hadoop.thirdparty.com.google.common.base.Preconditions;
 
 /** Verifier used by AUTH_NONE. */
 public class VerifierNone extends Verifier {
