@@ -29,7 +29,7 @@ import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.ConcurrentSkipListMap;
 
-import com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
 import org.apache.hadoop.classification.InterfaceAudience;
 import org.apache.hadoop.classification.InterfaceStability;
 import org.apache.hadoop.conf.Configuration;
@@ -53,7 +53,7 @@ import org.apache.hadoop.util.ReflectionUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.google.protobuf.ByteString;
+import org.apache.hadoop.thirdparty.protobuf.ByteString;
 
 /**
  * This class allows us to manage and multiplex between storages local to
@@ -220,6 +220,11 @@ public class ProvidedStorageMap {
     return providedDescriptor.chooseRandom();
   }
 
+  @VisibleForTesting
+  public BlockAliasMap getAliasMap() {
+    return aliasMap;
+  }
+
   /**
    * Builder used for creating {@link LocatedBlocks} when a block is provided.
    */
