@@ -139,7 +139,7 @@ public class JspHelper {
         // filter
         ugi.setAuthenticationMethod(secureAuthMethod);
       }
-      if (doAsUserFromQuery != null) {
+      if (doAsUserFromQuery != null && !doAsUserFromQuery.equals(remoteUser)) {
         // create and attempt to authorize a proxy user
         ugi = UserGroupInformation.createProxyUser(doAsUserFromQuery, ugi);
         ProxyUsers.authorize(ugi, getRemoteAddr(request));
@@ -198,6 +198,9 @@ public class JspHelper {
     return remoteAddr;
   }
 
+  public static int getRemotePort(HttpServletRequest request) {
+    return request.getRemotePort();
+  }
 
   /**
    * Expected user name should be a short name.
