@@ -29,8 +29,8 @@ import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.util.Random;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.CommonConfigurationKeys;
 import org.apache.hadoop.fs.FSDataInputStream;
@@ -48,7 +48,8 @@ import org.junit.Test;
 
 /** Unit tests for permission */
 public class TestPermission {
-  public static final Log LOG = LogFactory.getLog(TestPermission.class);
+  public static final Logger LOG =
+      LoggerFactory.getLogger(TestPermission.class);
 
   final private static Path ROOT_PATH = new Path("/data");
   final private static Path CHILD_DIR1 = new Path(ROOT_PATH, "child1");
@@ -100,31 +101,18 @@ public class TestPermission {
     Configuration conf = new Configuration();
     FsPermission.setUMask(conf, perm);
     assertEquals(18, FsPermission.getUMask(conf).toShort());
-    
-    // Test 2 - old configuration key set with decimal 
-    // umask value should be handled
-    perm = new FsPermission((short)18);
-    conf = new Configuration();
-    conf.set(FsPermission.DEPRECATED_UMASK_LABEL, "18");
-    assertEquals(18, FsPermission.getUMask(conf).toShort());
-    
-    // Test 3 - old configuration key overrides the new one
-    conf = new Configuration();
-    conf.set(FsPermission.DEPRECATED_UMASK_LABEL, "18");
-    conf.set(FsPermission.UMASK_LABEL, "000");
-    assertEquals(18, FsPermission.getUMask(conf).toShort());
-    
-    // Test 4 - new configuration key is handled
+
+    // Test 2 - new configuration key is handled
     conf = new Configuration();
     conf.set(FsPermission.UMASK_LABEL, "022");
     assertEquals(18, FsPermission.getUMask(conf).toShort());
 
-    // Test 5 - equivalent valid umask
+    // Test 3 - equivalent valid umask
     conf = new Configuration();
     conf.set(FsPermission.UMASK_LABEL, "0022");
     assertEquals(18, FsPermission.getUMask(conf).toShort());
 
-    // Test 6 - invalid umask
+    // Test 4 - invalid umask
     conf = new Configuration();
     conf.set(FsPermission.UMASK_LABEL, "1222");
     try {
@@ -134,7 +122,7 @@ public class TestPermission {
      //pass, exception successfully trigger
     }
 
-    // Test 7 - invalid umask
+    // Test 5 - invalid umask
     conf = new Configuration();
     conf.set(FsPermission.UMASK_LABEL, "01222");
     try {
@@ -209,7 +197,7 @@ public class TestPermission {
   }
 
   @Test
-  public void testFilePermision() throws Exception {
+  public void testFilePermission() throws Exception {
     final Configuration conf = new HdfsConfiguration();
     conf.setBoolean(DFSConfigKeys.DFS_PERMISSIONS_ENABLED_KEY, true);
     MiniDFSCluster cluster = new MiniDFSCluster.Builder(conf).numDataNodes(3).build();
@@ -386,8 +374,8 @@ public class TestPermission {
       fail("Expect ACE when a non-super user tries to change owner");
     } catch (AccessControlException e) {
       assertThat(e.getMessage(), startsWith("User " +
-              userfs.getFileStatus(file).getOwner() +
-              " is not a super user (non-super user cannot change owner)"));
+          userfs.getFileStatus(file).getOwner() +
+          " is not a super user (non-super user cannot change owner)"));
     }
   }
 
@@ -412,8 +400,9 @@ public class TestPermission {
       fail("Expect ACE or FNFE when a non-super user tries to change owner " +
           "for a non-existent file");
     } catch (AccessControlException e) {
-      assertThat(e.getMessage(), startsWith(
-          "Non-super user cannot change owner"));
+      assertThat(e.getMessage(), startsWith("User " +
+          userfs.getFileStatus(file).getOwner() +
+          " is not a super user (non-super user cannot change owner)"));
     } catch (FileNotFoundException e) {
     }
   }
@@ -423,6 +412,10 @@ public class TestPermission {
       fs.mkdirs(p);
       return true;
     } catch(AccessControlException e) {
+      // We check that AccessControlExceptions contain absolute paths.
+      Path parent = p.getParent();
+      assertTrue(parent.isUriPathAbsolute());
+      assertTrue(e.getMessage().contains(parent.toString()));
       return false;
     }
   }
@@ -432,6 +425,9 @@ public class TestPermission {
       fs.create(p);
       return true;
     } catch(AccessControlException e) {
+      Path parent = p.getParent();
+      assertTrue(parent.isUriPathAbsolute());
+      assertTrue(e.getMessage().contains(parent.toString()));
       return false;
     }
   }
@@ -441,6 +437,8 @@ public class TestPermission {
       fs.open(p);
       return true;
     } catch(AccessControlException e) {
+      assertTrue(p.isUriPathAbsolute());
+      assertTrue(e.getMessage().contains(p.toString()));
       return false;
     }
   }
@@ -451,6 +449,9 @@ public class TestPermission {
       fs.rename(src, dst);
       return true;
     } catch(AccessControlException e) {
+      Path parent = dst.getParent();
+      assertTrue(parent.isUriPathAbsolute());
+      assertTrue(e.getMessage().contains(parent.toString()));
       return false;
     }
   }
