@@ -232,7 +232,7 @@ public class GenericMRLoadGenerator extends Configured implements Tool {
     }
   }
 
-  static enum Counters { RECORDS_WRITTEN, BYTES_WRITTEN }
+  enum Counters { RECORDS_WRITTEN, BYTES_WRITTEN }
 
   static class RandomMapOutput extends MapReduceBase
       implements Mapper<Text,Text,Text,Text> {
