@@ -18,7 +18,6 @@
 
 package org.apache.hadoop.fs.s3a.impl;
 
-import javax.annotation.Nullable;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
@@ -26,7 +25,6 @@ import java.util.concurrent.CompletableFuture;
 import java.util.stream.Collectors;
 
 import com.amazonaws.services.s3.model.DeleteObjectsRequest;
-import com.amazonaws.services.s3.model.DeleteObjectsResult;
 import org.apache.hadoop.thirdparty.com.google.common.util.concurrent.ListeningExecutorService;
 import org.apache.hadoop.thirdparty.com.google.common.util.concurrent.MoreExecutors;
 import org.slf4j.Logger;
@@ -40,10 +38,6 @@ import org.apache.hadoop.fs.s3a.Retries;
 import org.apache.hadoop.fs.s3a.S3AFileStatus;
 import org.apache.hadoop.fs.s3a.S3ALocatedFileStatus;
 import org.apache.hadoop.fs.s3a.Tristate;
-import org.apache.hadoop.fs.s3a.s3guard.BulkOperationState;
-import org.apache.hadoop.fs.s3a.s3guard.MetadataStore;
-import org.apache.hadoop.fs.s3a.s3guard.S3Guard;
-import org.apache.hadoop.io.IOUtils;
 import org.apache.hadoop.util.DurationInfo;
 
 import static org.apache.hadoop.fs.store.audit.AuditingFunctions.callableWithinAuditSpan;
@@ -53,69 +47,22 @@ import static org.apache.hadoop.fs.s3a.impl.CallableSupplier.submit;
 
 /**
  * Implementation of the delete() operation.
- * <p>
- * How S3Guard/Store inconsistency is handled:
- * <ol>
- *   <li>
- *     The list operation does not ask for tombstone markers; objects
- *     under tombstones will be found and deleted.
- *     The {@code extraFilesDeleted} counter will be incremented here.
- *   </li>
- *   <li>
- *     That may result in recently deleted files being found and
- *     duplicate delete requests issued. This is mostly harmless.
- *   </li>
- *   <li>
- *     If a path is considered authoritative on the client, so only S3Guard
- *     is used for listings, we wrap up the delete with a scan of raw S3.
- *     This will find and eliminate OOB additions.
- *   </li>
- *   <li>
- *     Exception 1: simple directory markers of the form PATH + "/".
- *     These are treated as a signal that there are no children; no
- *     listing is made.
- *   </li>
- *   <li>
- *     Exception 2: delete(path, true) where path has a tombstone in S3Guard.
- *     Here the delete is downgraded to a no-op even before this operation
- *     is created. Thus: no listings of S3.
- *   </li>
- * </ol>
- * If this class is logged at debug, requests will be audited:
- * the response to a bulk delete call will be reviewed to see if there
- * were fewer files deleted than requested; that will be printed
- * at WARN level. This is independent of handling rejected delete
- * requests which raise exceptions -those are processed lower down.
- * <p>
- * Performance tuning:
- * <p>
- * The operation to POST a delete request (or issue many individual
- * DELETE calls) then update the S3Guard table is done in an async
- * operation so that it can overlap with the LIST calls for data.
- * However, only one single operation is queued at a time.
- * <p>
- * Executing more than one batch delete is possible, it just
- * adds complexity in terms of error handling as well as in
- * the datastructures used to track outstanding operations.
+ * This issues only one bulk delete at a time,
+ * intending to update S3Guard after every request succeeded.
+ * Now that S3Guard has been removed, it
+ * would be possible to issue multiple delete calls
+ * in parallel.
  * If this is done, then it may be good to experiment with different
  * page sizes. The default value is
  * {@link InternalConstants#MAX_ENTRIES_TO_DELETE}, the maximum a single
  * POST permits.
  * <p>
- * 1. Smaller pages executed in parallel may have different
- * performance characteristics when deleting very large directories,
- * because it will be the DynamoDB calls which will come to dominate.
+ * Smaller pages executed in parallel may have different
+ * performance characteristics when deleting very large directories.
  * Any exploration of options here MUST be done with performance
- * measurements taken from test runs in EC2 against local DDB and S3 stores,
+ * measurements taken from test runs in EC2 against local S3 stores,
  * so as to ensure network latencies do not skew the results.
- * <p>
- * 2. Note that as the DDB thread/connection pools will be shared across
- * all active delete operations, speedups will be minimal unless
- * those pools are large enough to cope the extra load.
- * <p>
- * There are also some opportunities to explore in
- * {@code DynamoDBMetadataStore} with batching delete requests
- * in the DDB APIs.
+
  */
 public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
 
@@ -142,11 +89,6 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
    */
   private final int pageSize;
 
-  /**
-   * Metastore -never null but may be the NullMetadataStore.
-   */
-  private final MetadataStore metadataStore;
-
   /**
    * Executor for async operations.
    */
@@ -157,35 +99,16 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
    */
   private List<DeleteEntry> keys;
 
-  /**
-   * List of paths built up for incremental deletion on tree delete.
-   * At the end of the entire delete the full tree is scanned in S3Guard
-   * and tombstones added. For this reason this list of paths <i>must not</i>
-   * include directory markers, as that will break the scan.
-   */
-  private List<Path> paths;
-
   /**
    * The single async delete operation, or null.
    */
   private CompletableFuture<Void> deleteFuture;
 
-  /**
-   * Bulk Operation state if this is a bulk operation.
-   */
-  private BulkOperationState operationState;
-
   /**
    * Counter of deleted files.
    */
   private long filesDeleted;
 
-  /**
-   * Counter of files found in the S3 Store during a raw scan of the store
-   * after the previous listing was in auth-mode.
-   */
-  private long extraFilesDeleted;
-
   /**
    * Constructor.
    * @param context store context
@@ -208,7 +131,6 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
             && pageSize <= InternalConstants.MAX_ENTRIES_TO_DELETE,
         "page size out of range: %s", pageSize);
     this.pageSize = pageSize;
-    metadataStore = context.getMetadataStore();
     executor = MoreExecutors.listeningDecorator(
         context.createThrottledExecutor(1));
   }
@@ -217,10 +139,6 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
     return filesDeleted;
   }
 
-  public long getExtraFilesDeleted() {
-    return extraFilesDeleted;
-  }
-
   /**
    * Delete a file or directory tree.
    * <p>
@@ -230,10 +148,14 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
    * Only one delete at a time is submitted, however, to reduce the
    * complexity of recovering from failures.
    * <p>
-   * The DynamoDB store deletes paths in parallel itself, so that
-   * potentially slow part of the process is somewhat speeded up.
-   * The extra parallelization here is to list files from the store/DDB while
-   * that delete operation is in progress.
+   * With S3Guard removed, the problem of updating any
+   * DynamoDB store has gone away -delete calls could now be issued
+   * in parallel.
+   * However, rate limiting may be required to keep write load
+   * below the throttling point. Every entry in a single
+   * bulk delete call counts as a single write request -overloading
+   * an S3 partition with delete calls has been a problem in
+   * the past.
    *
    * @return true, except in the corner cases of root directory deletion
    * @throws PathIsNotEmptyDirectoryException if the path is a dir and this
@@ -294,9 +216,7 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
    * Delete a directory tree.
    * <p>
    * This is done by asking the filesystem for a list of all objects under
-   * the directory path, without using any S3Guard tombstone markers to hide
-   * objects which may be returned in S3 listings but which are considered
-   * deleted.
+   * the directory path.
    * <p>
    * Once the first {@link #pageSize} worth of objects has been listed, a batch
    * delete is queued for execution in a separate thread; subsequent batches
@@ -304,9 +224,6 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
    * being deleted in the separate thread.
    * <p>
    * After all listed objects are queued for deletion,
-   * if the path is considered authoritative in the client, a final scan
-   * of S3 <i>without S3Guard</i> is executed, so as to find and delete
-   * any out-of-band objects in the tree.
    * @param path directory path
    * @param dirKey directory key
    * @throws IOException failure
@@ -314,12 +231,7 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
   @Retries.RetryTranslated
   protected void deleteDirectoryTree(final Path path,
       final String dirKey) throws IOException {
-    // create an operation state so that the store can manage the bulk
-    // operation if it needs to
-    operationState = S3Guard.initiateBulkWrite(
-        metadataStore,
-        BulkOperationState.OperationType.Delete,
-        path);
+
     try (DurationInfo ignored =
              new DurationInfo(LOG, false, "deleting %s", dirKey)) {
 
@@ -327,11 +239,11 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
       resetDeleteList();
       deleteFuture = null;
 
-      // list files including any under tombstones through S3Guard
+      // list files
       LOG.debug("Getting objects for directory prefix {} to delete", dirKey);
       final RemoteIterator<S3ALocatedFileStatus> locatedFiles =
           callbacks.listFilesAndDirectoryMarkers(path, status,
-              false, true);
+              true);
 
       // iterate through and delete. The next() call will block when a new S3
       // page is required; this any active delete submitted to the executor
@@ -345,50 +257,6 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
       submitNextBatch();
       maybeAwaitCompletion(deleteFuture);
 
-      // if s3guard is authoritative we follow up with a bulk list and
-      // delete process on S3 this helps recover from any situation where S3
-      // and S3Guard have become inconsistent.
-      // This is only needed for auth paths; by performing the previous listing
-      // without tombstone filtering, any files returned by the non-auth
-      // S3 list which were hidden under tombstones will have been found
-      // and deleted.
-
-      if (callbacks.allowAuthoritative(path)) {
-        LOG.debug("Path is authoritatively guarded;"
-            + " listing files on S3 for completeness");
-        // let the ongoing delete finish to avoid duplicates
-        final RemoteIterator<S3AFileStatus> objects =
-            callbacks.listObjects(path, dirKey);
-
-        // iterate through and delete. The next() call will block when a new S3
-        // page is required; this any active delete submitted to the executor
-        // will run in parallel with this.
-        while (objects.hasNext()) {
-          // get the next entry in the listing.
-          extraFilesDeleted++;
-          S3AFileStatus next = objects.next();
-          LOG.debug("Found Unlisted entry {}", next);
-          queueForDeletion(deletionKey(next), null,
-              next.isDirectory());
-        }
-        if (extraFilesDeleted > 0) {
-          LOG.debug("Raw S3 Scan found {} extra file(s) to delete",
-              extraFilesDeleted);
-          // there is no more data:
-          // await any ongoing operation
-          submitNextBatch();
-          maybeAwaitCompletion(deleteFuture);
-        }
-      }
-
-      // final cleanup of the directory tree in the metastore, including the
-      // directory entry itself.
-      try (DurationInfo ignored2 =
-               new DurationInfo(LOG, false, "Delete metastore")) {
-        metadataStore.deleteSubtree(path, operationState);
-      }
-    } finally {
-      IOUtils.cleanupWithLogger(LOG, operationState);
     }
     LOG.debug("Delete \"{}\" completed; deleted {} objects", path,
         filesDeleted);
@@ -412,7 +280,7 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
    */
   private void queueForDeletion(
       final S3AFileStatus stat) throws IOException {
-    queueForDeletion(deletionKey(stat), stat.getPath(), stat.isDirectory());
+    queueForDeletion(deletionKey(stat), stat.isDirectory());
   }
 
   /**
@@ -422,21 +290,13 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
    * complete.
    *
    * @param key key to delete
-   * @param deletePath nullable path of the key
    * @param isDirMarker is the entry a directory?
    * @throws IOException failure of the previous batch of deletions.
    */
   private void queueForDeletion(final String key,
-      @Nullable final Path deletePath,
       boolean isDirMarker) throws IOException {
     LOG.debug("Adding object to delete: \"{}\"", key);
     keys.add(new DeleteEntry(key, isDirMarker));
-    if (deletePath != null) {
-      if (!isDirMarker) {
-        paths.add(deletePath);
-      }
-    }
-
     if (keys.size() == pageSize) {
       submitNextBatch();
     }
@@ -455,7 +315,7 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
     maybeAwaitCompletion(deleteFuture);
 
     // delete the current page of keys and paths
-    deleteFuture = submitDelete(keys, paths);
+    deleteFuture = submitDelete(keys);
     // reset the references so a new list can be built up.
     resetDeleteList();
   }
@@ -466,7 +326,6 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
    */
   private void resetDeleteList() {
     keys = new ArrayList<>(pageSize);
-    paths = new ArrayList<>(pageSize);
   }
 
   /**
@@ -484,34 +343,28 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
       throws IOException {
     LOG.debug("delete: {} {}", (isFile ? "file" : "dir marker"), key);
     filesDeleted++;
-    callbacks.deleteObjectAtPath(path, key, isFile, operationState);
+    callbacks.deleteObjectAtPath(path, key, isFile);
   }
 
   /**
-   * Delete a single page of keys and optionally the metadata.
-   * For a large page, it is the metadata size which dominates.
-   * Its possible to invoke this with empty lists of keys or paths.
-   * If both lists are empty no work is submitted and null is returned.
+   * Delete a single page of keys.
+   * If the list is empty no work is submitted and null is returned.
    *
    * @param keyList keys to delete.
-   * @param pathList paths to update the metastore with.
    * @return the submitted future or null
    */
   private CompletableFuture<Void> submitDelete(
-      final List<DeleteEntry> keyList,
-      final List<Path> pathList) {
+      final List<DeleteEntry> keyList) {
 
-    if (keyList.isEmpty() && pathList.isEmpty()) {
+    if (keyList.isEmpty()) {
       return null;
     }
     filesDeleted += keyList.size();
     return submit(executor,
         callableWithinAuditSpan(
             getAuditSpan(), () -> {
-              asyncDeleteAction(operationState,
-                  keyList,
-                  pathList,
-                  LOG.isDebugEnabled());
+              asyncDeleteAction(
+                  keyList);
               return null;
             }));
   }
@@ -520,26 +373,17 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
    * The action called in the asynchronous thread to delete
    * the keys from S3 and paths from S3Guard.
    *
-   * @param state ongoing operation state
    * @param keyList keys to delete.
-   * @param pathList paths to update the metastore with.
-   * @param auditDeletedKeys should the results be audited and undeleted
    * entries logged?
    * @throws IOException failure
    */
   @Retries.RetryTranslated
   private void asyncDeleteAction(
-      final BulkOperationState state,
-      final List<DeleteEntry> keyList,
-      final List<Path> pathList,
-      final boolean auditDeletedKeys)
+      final List<DeleteEntry> keyList)
       throws IOException {
-    List<DeleteObjectsResult.DeletedObject> deletedObjects = new ArrayList<>();
     try (DurationInfo ignored =
              new DurationInfo(LOG, false,
                  "Delete page of %d keys", keyList.size())) {
-      DeleteObjectsResult result = null;
-      List<Path> undeletedObjects = new ArrayList<>();
       if (!keyList.isEmpty()) {
         // first delete the files.
         List<DeleteObjectsRequest.KeyVersion> files = keyList.stream()
@@ -547,66 +391,35 @@ public class DeleteOperation extends ExecutingStoreOperation<Boolean> {
             .map(e -> e.keyVersion)
             .collect(Collectors.toList());
         LOG.debug("Deleting of {} file objects", files.size());
-        result = Invoker.once("Remove S3 Files",
+        Invoker.once("Remove S3 Files",
             status.getPath().toString(),
             () -> callbacks.removeKeys(
                 files,
-                false,
-                undeletedObjects,
-                state,
-                !auditDeletedKeys));
-        if (result != null) {
-          deletedObjects.addAll(result.getDeletedObjects());
-        }
+                false
+            ));
         // now the dirs
         List<DeleteObjectsRequest.KeyVersion> dirs = keyList.stream()
             .filter(e -> e.isDirMarker)
             .map(e -> e.keyVersion)
             .collect(Collectors.toList());
         LOG.debug("Deleting of {} directory markers", dirs.size());
-        // This is invoked with deleteFakeDir = true, so
-        // S3Guard is not updated.
-        result = Invoker.once("Remove S3 Dir Markers",
+        // This is invoked with deleteFakeDir.
+        Invoker.once("Remove S3 Dir Markers",
             status.getPath().toString(),
             () -> callbacks.removeKeys(
                 dirs,
-                true,
-                undeletedObjects,
-                state,
-                !auditDeletedKeys));
-        if (result != null) {
-          deletedObjects.addAll(result.getDeletedObjects());
-        }
-      }
-      if (!pathList.isEmpty()) {
-        // delete file paths only. This stops tombstones
-        // being added until the final directory cleanup
-        // (HADOOP-17244)
-        metadataStore.deletePaths(pathList, state);
-      }
-      if (auditDeletedKeys) {
-        // audit the deleted keys
-        if (deletedObjects.size() != keyList.size()) {
-          // size mismatch
-          LOG.warn("Size mismatch in deletion operation. "
-                  + "Expected count of deleted files: {}; "
-                  + "actual: {}",
-              keyList.size(), deletedObjects.size());
-          // strip out the deleted keys
-          for (DeleteObjectsResult.DeletedObject del : deletedObjects) {
-            keyList.removeIf(kv -> kv.getKey().equals(del.getKey()));
-          }
-          for (DeleteEntry kv : keyList) {
-            LOG.debug("{}", kv.getKey());
-          }
-        }
+                true
+            ));
       }
     }
   }
 
   /**
-   * Deletion entry; dir marker state is tracked to control S3Guard
-   * update policy.
+   * Deletion entry; dir marker state is tracked to allow
+   * delete requests to be split into file
+   * and marker delete phases.
+   * Without S3Guard, the split is only used
+   * to choose which statistics to update.
    */
   private static final class DeleteEntry {
     private final DeleteObjectsRequest.KeyVersion keyVersion;
