@@ -223,6 +223,15 @@ public class RMNodeWrapper implements RMNode {
     return null;
   }
 
+  @Override
+  public boolean isUpdatedCapability() {
+    return false;
+  }
+
+  @Override
+  public void resetUpdatedCapability() {
+  }
+
   @Override
   public long calculateHeartBeatInterval(
       long defaultInterval, long minInterval, long maxInterval,
