@@ -43,20 +43,18 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import javax.net.ssl.HttpsURLConnection;
 import javax.net.ssl.SSLSocketFactory;
 
-import com.google.common.annotations.VisibleForTesting;
-import com.google.common.cache.Cache;
-import com.google.common.cache.CacheBuilder;
+import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.thirdparty.com.google.common.cache.Cache;
+import org.apache.hadoop.thirdparty.com.google.common.cache.CacheBuilder;
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.GnuParser;
 import org.apache.commons.cli.HelpFormatter;
 import org.apache.commons.cli.Options;
 import org.apache.commons.cli.ParseException;
 import org.apache.commons.io.IOUtils;
-import org.apache.commons.lang.StringUtils;
-import org.apache.commons.lang.time.DateFormatUtils;
-import org.apache.commons.lang.time.DurationFormatUtils;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.commons.lang3.StringUtils;
+import org.apache.commons.lang3.time.DateFormatUtils;
+import org.apache.commons.lang3.time.DurationFormatUtils;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.http.HttpConfig.Policy;
 import org.apache.hadoop.security.UserGroupInformation;
@@ -78,12 +76,15 @@ import org.apache.hadoop.yarn.exceptions.YarnException;
 import org.apache.hadoop.yarn.webapp.util.WebAppUtils;
 import org.codehaus.jettison.json.JSONException;
 import org.codehaus.jettison.json.JSONObject;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 public class TopCLI extends YarnCLI {
 
   private static final String CLUSTER_INFO_URL = "/ws/v1/cluster/info";
 
-  private static final Log LOG = LogFactory.getLog(TopCLI.class);
+  private static final Logger LOG = LoggerFactory
+          .getLogger(TopCLI.class);
   private String CLEAR = "\u001b[2J";
   private String CLEAR_LINE = "\u001b[2K";
   private String SET_CURSOR_HOME = "\u001b[H";
@@ -338,9 +339,11 @@ public class TopCLI extends YarnCLI {
     int totalNodes;
     int runningNodes;
     int unhealthyNodes;
+    int decommissioningNodes;
     int decommissionedNodes;
     int lostNodes;
     int rebootedNodes;
+    int shutdownNodes;
   }
 
   private static class QueueMetrics {
@@ -462,6 +465,7 @@ public class TopCLI extends YarnCLI {
       LOG.error("Unable to parse options", e);
       return 1;
     }
+    createAndStartYarnClient();
     setAppsHeader();
 
     Thread keyboardMonitor = new KeyboardMonitor();
@@ -694,6 +698,8 @@ public class TopCLI extends YarnCLI {
       return nodeInfo;
     }
 
+    nodeInfo.decommissioningNodes =
+        yarnClusterMetrics.getNumDecommissioningNodeManagers();
     nodeInfo.decommissionedNodes =
         yarnClusterMetrics.getNumDecommissionedNodeManagers();
     nodeInfo.totalNodes = yarnClusterMetrics.getNumNodeManagers();
@@ -701,6 +707,7 @@ public class TopCLI extends YarnCLI {
     nodeInfo.lostNodes = yarnClusterMetrics.getNumLostNodeManagers();
     nodeInfo.unhealthyNodes = yarnClusterMetrics.getNumUnhealthyNodeManagers();
     nodeInfo.rebootedNodes = yarnClusterMetrics.getNumRebootedNodeManagers();
+    nodeInfo.shutdownNodes = yarnClusterMetrics.getNumShutdownNodeManagers();
     return nodeInfo;
   }
 
@@ -866,45 +873,50 @@ public class TopCLI extends YarnCLI {
         TimeUnit.MILLISECONDS.toMinutes(uptime)
             - TimeUnit.HOURS.toMinutes(TimeUnit.MILLISECONDS.toHours(uptime));
     String uptimeStr = String.format("%dd, %d:%d", days, hours, minutes);
-    String currentTime = DateFormatUtils.ISO_TIME_NO_T_FORMAT.format(now);
+    String currentTime = DateFormatUtils.ISO_8601_EXTENDED_TIME_FORMAT
+        .format(now);
 
-    ret.append(CLEAR_LINE);
-    ret.append(limitLineLength(String.format(
-      "YARN top - %s, up %s, %d active users, queue(s): %s%n", currentTime,
-      uptimeStr, queueMetrics.activeUsers, queue), terminalWidth, true));
+    ret.append(CLEAR_LINE)
+        .append(limitLineLength(String.format(
+            "YARN top - %s, up %s, %d active users, queue(s): %s%n",
+            currentTime, uptimeStr, queueMetrics.activeUsers, queue),
+            terminalWidth, true));
 
-    ret.append(CLEAR_LINE);
-    ret.append(limitLineLength(String.format(
-      "NodeManager(s): %d total, %d active, %d unhealthy, %d decommissioned,"
-          + " %d lost, %d rebooted%n", nodes.totalNodes, nodes.runningNodes,
-      nodes.unhealthyNodes, nodes.decommissionedNodes, nodes.lostNodes,
-      nodes.rebootedNodes), terminalWidth, true));
+    ret.append(CLEAR_LINE)
+        .append(limitLineLength(String.format(
+            "NodeManager(s)"
+                + ": %d total, %d active, %d unhealthy, %d decommissioning,"
+                + " %d decommissioned, %d lost, %d rebooted, %d shutdown%n",
+            nodes.totalNodes, nodes.runningNodes, nodes.unhealthyNodes,
+            nodes.decommissioningNodes, nodes.decommissionedNodes, nodes.lostNodes,
+            nodes.rebootedNodes, nodes.shutdownNodes), terminalWidth, true));
 
-    ret.append(CLEAR_LINE);
-    ret.append(limitLineLength(String.format(
-        "Queue(s) Applications: %d running, %d submitted, %d pending,"
-            + " %d completed, %d killed, %d failed%n", queueMetrics.appsRunning,
-        queueMetrics.appsSubmitted, queueMetrics.appsPending,
-        queueMetrics.appsCompleted, queueMetrics.appsKilled,
-        queueMetrics.appsFailed), terminalWidth, true));
+    ret.append(CLEAR_LINE)
+        .append(limitLineLength(String.format(
+            "Queue(s) Applications: %d running, %d submitted, %d pending,"
+                + " %d completed, %d killed, %d failed%n",
+            queueMetrics.appsRunning, queueMetrics.appsSubmitted,
+            queueMetrics.appsPending, queueMetrics.appsCompleted,
+            queueMetrics.appsKilled, queueMetrics.appsFailed), terminalWidth,
+            true));
 
-    ret.append(CLEAR_LINE);
-    ret.append(limitLineLength(String.format("Queue(s) Mem(GB): %d available,"
-        + " %d allocated, %d pending, %d reserved%n",
-      queueMetrics.availableMemoryGB, queueMetrics.allocatedMemoryGB,
-      queueMetrics.pendingMemoryGB, queueMetrics.reservedMemoryGB),
-      terminalWidth, true));
+    ret.append(CLEAR_LINE)
+        .append(limitLineLength(String.format("Queue(s) Mem(GB): %d available,"
+            + " %d allocated, %d pending, %d reserved%n",
+            queueMetrics.availableMemoryGB, queueMetrics.allocatedMemoryGB,
+            queueMetrics.pendingMemoryGB, queueMetrics.reservedMemoryGB),
+            terminalWidth, true));
 
-    ret.append(CLEAR_LINE);
-    ret.append(limitLineLength(String.format("Queue(s) VCores: %d available,"
-        + " %d allocated, %d pending, %d reserved%n",
-      queueMetrics.availableVCores, queueMetrics.allocatedVCores,
-      queueMetrics.pendingVCores, queueMetrics.reservedVCores), terminalWidth,
-      true));
+    ret.append(CLEAR_LINE)
+        .append(limitLineLength(String.format("Queue(s) VCores: %d available,"
+            + " %d allocated, %d pending, %d reserved%n",
+            queueMetrics.availableVCores, queueMetrics.allocatedVCores,
+            queueMetrics.pendingVCores, queueMetrics.reservedVCores),
+            terminalWidth, true));
 
-    ret.append(CLEAR_LINE);
-    ret.append(limitLineLength(String.format(
-        "Queue(s) Containers: %d allocated, %d pending, %d reserved%n",
+    ret.append(CLEAR_LINE)
+        .append(limitLineLength(String.format(
+            "Queue(s) Containers: %d allocated, %d pending, %d reserved%n",
             queueMetrics.allocatedContainers, queueMetrics.pendingContainers,
             queueMetrics.reservedContainers), terminalWidth, true));
     return ret.toString();
@@ -977,8 +989,8 @@ public class TopCLI extends YarnCLI {
       System.out.println("Delay: " + (refreshPeriod / 1000)
           + " secs; Secure mode: " + UserGroupInformation.isSecurityEnabled());
       System.out.println("");
-      System.out.println("  s + Enter : Select sort field");
-      System.out.println("  f + Enter : Select fields to display");
+      System.out.println("  s + Enter: Select sort field");
+      System.out.println("  f + Enter: Select fields to display");
       System.out.println("  R + Enter: Reverse current sort order");
       System.out.println("  h + Enter: Display this screen");
       System.out.println("  q + Enter: Quit");
@@ -1032,7 +1044,8 @@ public class TopCLI extends YarnCLI {
     }
   }
 
-  protected void showTopScreen() {
+  @VisibleForTesting
+  void showTopScreen() {
     List<ApplicationInformation> appsInfo = new ArrayList<>();
     List<ApplicationReport> apps;
     try {
