@@ -295,7 +295,7 @@ public class TestRMFailover extends ClientBaseWithFixes {
     assertEquals(redirectURL,rm1Url + "/metrics");
 
 
-    // standby RM links /conf, /stacks, /logLevel, /static, /logs, /jmx
+    // standby RM links /conf, /stacks, /logLevel, /static, /logs, /jmx, /prom
     // /cluster/cluster as well as webService
     // /ws/v1/cluster/info should not be redirected to active RM
     redirectURL = getRedirectURL(rm2Url + "/cluster/cluster");
@@ -319,6 +319,9 @@ public class TestRMFailover extends ClientBaseWithFixes {
     redirectURL = getRedirectURL(rm2Url + "/jmx?param1=value1+x&param2=y");
     assertNull(redirectURL);
 
+    redirectURL = getRedirectURL(rm2Url + "/prom");
+    assertNull(redirectURL);
+
     redirectURL = getRedirectURL(rm2Url + "/ws/v1/cluster/info");
     assertNull(redirectURL);
 
