@@ -21,11 +21,11 @@ package org.apache.hadoop.yarn.api.records.impl.pb;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.TreeSet;
 
 import org.apache.hadoop.classification.InterfaceAudience.Private;
 import org.apache.hadoop.classification.InterfaceStability.Unstable;
@@ -49,9 +49,9 @@ import org.apache.hadoop.yarn.proto.YarnProtos.PriorityProto;
 import org.apache.hadoop.yarn.proto.YarnProtos.ReservationIdProto;
 import org.apache.hadoop.yarn.proto.YarnProtos.ResourceProto;
 import org.apache.hadoop.yarn.proto.YarnProtos.ResourceRequestProto;
+import org.apache.hadoop.yarn.proto.YarnProtos.StringStringMapProto;
 
-import com.google.common.base.CharMatcher;
-import com.google.protobuf.TextFormat;
+import org.apache.hadoop.thirdparty.protobuf.TextFormat;
 
 @Private
 @Unstable
@@ -71,6 +71,7 @@ extends ApplicationSubmissionContext {
   private LogAggregationContext logAggregationContext = null;
   private ReservationId reservationId = null;
   private Map<ApplicationTimeoutType, Long> applicationTimeouts = null;
+  private Map<String, String> schedulingProperties = null;
 
   public ApplicationSubmissionContextPBImpl() {
     builder = ApplicationSubmissionContextProto.newBuilder();
@@ -82,7 +83,7 @@ extends ApplicationSubmissionContext {
     viaProto = true;
   }
   
-  public ApplicationSubmissionContextProto getProto() {
+  public synchronized ApplicationSubmissionContextProto getProto() {
       mergeLocalToProto();
     proto = viaProto ? proto : builder.build();
     viaProto = true;
@@ -141,6 +142,9 @@ extends ApplicationSubmissionContext {
     if (this.applicationTimeouts != null) {
       addApplicationTimeouts();
     }
+    if (this.schedulingProperties != null) {
+      addApplicationSchedulingProperties();
+    }
   }
 
   private void mergeLocalToProto() {
@@ -159,7 +163,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public Priority getPriority() {
+  public synchronized Priority getPriority() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (this.priority != null) {
       return this.priority;
@@ -172,7 +176,7 @@ extends ApplicationSubmissionContext {
   }
   
   @Override
-  public void setPriority(Priority priority) {
+  public synchronized void setPriority(Priority priority) {
     maybeInitBuilder();
     if (priority == null)
       builder.clearPriority();
@@ -180,7 +184,7 @@ extends ApplicationSubmissionContext {
   }
   
   @Override
-  public ApplicationId getApplicationId() {
+  public synchronized ApplicationId getApplicationId() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (this.applicationId != null) {
       return applicationId;
@@ -193,7 +197,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void setApplicationId(ApplicationId applicationId) {
+  public synchronized void setApplicationId(ApplicationId applicationId) {
     maybeInitBuilder();
     if (applicationId == null)
       builder.clearApplicationId();
@@ -201,7 +205,7 @@ extends ApplicationSubmissionContext {
   }
   
   @Override
-  public String getApplicationName() {
+  public synchronized String getApplicationName() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (!p.hasApplicationName()) {
       return null;
@@ -210,7 +214,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void setApplicationName(String applicationName) {
+  public synchronized void setApplicationName(String applicationName) {
     maybeInitBuilder();
     if (applicationName == null) {
       builder.clearApplicationName();
@@ -220,7 +224,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public String getQueue() {
+  public synchronized String getQueue() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (!p.hasQueue()) {
       return null;
@@ -229,7 +233,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public String getApplicationType() {
+  public synchronized String getApplicationType() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (!p.hasApplicationType()) {
       return null;
@@ -242,18 +246,18 @@ extends ApplicationSubmissionContext {
       return;
     }
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
-    this.applicationTags = new HashSet<String>();
+    this.applicationTags = new TreeSet<>();
     this.applicationTags.addAll(p.getApplicationTagsList());
   }
 
   @Override
-  public Set<String> getApplicationTags() {
+  public synchronized Set<String> getApplicationTags() {
     initApplicationTags();
     return this.applicationTags;
   }
 
   @Override
-  public void setQueue(String queue) {
+  public synchronized void setQueue(String queue) {
     maybeInitBuilder();
     if (queue == null) {
       builder.clearQueue();
@@ -263,7 +267,7 @@ extends ApplicationSubmissionContext {
   }
   
   @Override
-  public void setApplicationType(String applicationType) {
+  public synchronized void setApplicationType(String applicationType) {
     maybeInitBuilder();
     if (applicationType == null) {
       builder.clearApplicationType();
@@ -273,7 +277,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void setApplicationTags(Set<String> tags) {
+  public synchronized void setApplicationTags(Set<String> tags) {
     maybeInitBuilder();
     if (tags == null || tags.isEmpty()) {
       builder.clearApplicationTags();
@@ -281,14 +285,14 @@ extends ApplicationSubmissionContext {
       return;
     }
     // Convert applicationTags to lower case and add
-    this.applicationTags = new HashSet<String>();
+    this.applicationTags = new TreeSet<>();
     for (String tag : tags) {
       this.applicationTags.add(StringUtils.toLowerCase(tag));
     }
   }
 
   @Override
-  public ContainerLaunchContext getAMContainerSpec() {
+  public synchronized ContainerLaunchContext getAMContainerSpec() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (this.amContainer != null) {
       return amContainer;
@@ -301,7 +305,8 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void setAMContainerSpec(ContainerLaunchContext amContainer) {
+  public synchronized void
+      setAMContainerSpec(ContainerLaunchContext amContainer) {
     maybeInitBuilder();
     if (amContainer == null) {
       builder.clearAmContainerSpec();
@@ -310,44 +315,44 @@ extends ApplicationSubmissionContext {
   }
   
   @Override
-  public boolean getUnmanagedAM() {
+  public synchronized boolean getUnmanagedAM() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     return p.getUnmanagedAm();
   }
   
   @Override
-  public void setUnmanagedAM(boolean value) {
+  public synchronized void setUnmanagedAM(boolean value) {
     maybeInitBuilder();
     builder.setUnmanagedAm(value);
   }
   
   @Override
-  public boolean getCancelTokensWhenComplete() {
+  public synchronized boolean getCancelTokensWhenComplete() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     //There is a default so cancelTokens should never be null
     return p.getCancelTokensWhenComplete();
   }
   
   @Override
-  public void setCancelTokensWhenComplete(boolean cancel) {
+  public synchronized void setCancelTokensWhenComplete(boolean cancel) {
     maybeInitBuilder();
     builder.setCancelTokensWhenComplete(cancel);
   }
 
   @Override
-  public int getMaxAppAttempts() {
+  public synchronized int getMaxAppAttempts() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     return p.getMaxAppAttempts();
   }
 
   @Override
-  public void setMaxAppAttempts(int maxAppAttempts) {
+  public synchronized void setMaxAppAttempts(int maxAppAttempts) {
     maybeInitBuilder();
     builder.setMaxAppAttempts(maxAppAttempts);
   }
 
   @Override
-  public Resource getResource() {
+  public synchronized Resource getResource() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (this.resource != null) {
       return this.resource;
@@ -360,7 +365,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void setResource(Resource resource) {
+  public synchronized void setResource(Resource resource) {
     maybeInitBuilder();
     if (resource == null) {
       builder.clearResource();
@@ -369,7 +374,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public ReservationId getReservationID() {
+  public synchronized ReservationId getReservationID() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (reservationId != null) {
       return reservationId;
@@ -382,7 +387,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void setReservationID(ReservationId reservationID) {
+  public synchronized void setReservationID(ReservationId reservationID) {
     maybeInitBuilder();
     if (reservationID == null) {
       builder.clearReservationId();
@@ -392,14 +397,14 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void
+  public synchronized void
       setKeepContainersAcrossApplicationAttempts(boolean keepContainers) {
     maybeInitBuilder();
     builder.setKeepContainersAcrossApplicationAttempts(keepContainers);
   }
 
   @Override
-  public boolean getKeepContainersAcrossApplicationAttempts() {
+  public synchronized boolean getKeepContainersAcrossApplicationAttempts() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     return p.getKeepContainersAcrossApplicationAttempts();
   }
@@ -457,7 +462,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public String getNodeLabelExpression() {
+  public synchronized String getNodeLabelExpression() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (!p.hasNodeLabelExpression()) {
       return null;
@@ -466,7 +471,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void setNodeLabelExpression(String labelExpression) {
+  public synchronized void setNodeLabelExpression(String labelExpression) {
     maybeInitBuilder();
     if (labelExpression == null) {
       builder.clearNodeLabelExpression();
@@ -477,7 +482,7 @@ extends ApplicationSubmissionContext {
   
   @Override
   @Deprecated
-  public ResourceRequest getAMContainerResourceRequest() {
+  public synchronized ResourceRequest getAMContainerResourceRequest() {
     List<ResourceRequest> reqs = getAMContainerResourceRequests();
     if (reqs == null || reqs.isEmpty()) {
       return null;
@@ -486,7 +491,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public List<ResourceRequest> getAMContainerResourceRequests() {
+  public synchronized List<ResourceRequest> getAMContainerResourceRequests() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (this.amResourceRequests != null) {
       return amResourceRequests;
@@ -501,7 +506,8 @@ extends ApplicationSubmissionContext {
 
   @Override
   @Deprecated
-  public void setAMContainerResourceRequest(ResourceRequest request) {
+  public synchronized void setAMContainerResourceRequest(
+      ResourceRequest request) {
     maybeInitBuilder();
     if (request == null) {
       builder.clearAmContainerResourceRequest();
@@ -510,7 +516,8 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void setAMContainerResourceRequests(List<ResourceRequest> requests) {
+  public synchronized void setAMContainerResourceRequests(
+      List<ResourceRequest> requests) {
     maybeInitBuilder();
     if (requests == null) {
       builder.clearAmContainerResourceRequest();
@@ -519,13 +526,13 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public long getAttemptFailuresValidityInterval() {
+  public synchronized long getAttemptFailuresValidityInterval() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     return p.getAttemptFailuresValidityInterval();
   }
 
   @Override
-  public void setAttemptFailuresValidityInterval(
+  public synchronized void setAttemptFailuresValidityInterval(
       long attemptFailuresValidityInterval) {
     maybeInitBuilder();
     builder.setAttemptFailuresValidityInterval(attemptFailuresValidityInterval);
@@ -542,7 +549,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public LogAggregationContext getLogAggregationContext() {
+  public synchronized LogAggregationContext getLogAggregationContext() {
     ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
     if (this.logAggregationContext != null) {
       return this.logAggregationContext;
@@ -555,7 +562,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void setLogAggregationContext(
+  public synchronized void setLogAggregationContext(
       LogAggregationContext logAggregationContext) {
     maybeInitBuilder();
     if (logAggregationContext == null)
@@ -572,7 +579,8 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public Map<ApplicationTimeoutType, Long> getApplicationTimeouts() {
+  public synchronized
+      Map<ApplicationTimeoutType, Long> getApplicationTimeouts() {
     initApplicationTimeout();
     return this.applicationTimeouts;
   }
@@ -594,7 +602,7 @@ extends ApplicationSubmissionContext {
   }
 
   @Override
-  public void setApplicationTimeouts(
+  public synchronized void setApplicationTimeouts(
       Map<ApplicationTimeoutType, Long> appTimeouts) {
     if (appTimeouts == null) {
       return;
@@ -643,4 +651,72 @@ extends ApplicationSubmissionContext {
         };
     this.builder.addAllApplicationTimeouts(values);
   }
-}  
+
+  private void addApplicationSchedulingProperties() {
+    maybeInitBuilder();
+    builder.clearApplicationSchedulingProperties();
+    if (this.schedulingProperties == null) {
+      return;
+    }
+    Iterable<? extends StringStringMapProto> values =
+        new Iterable<StringStringMapProto>() {
+
+      @Override
+      public Iterator<StringStringMapProto> iterator() {
+        return new Iterator<StringStringMapProto>() {
+          private Iterator<String> iterator = schedulingProperties.keySet()
+              .iterator();
+
+          @Override
+          public boolean hasNext() {
+            return iterator.hasNext();
+          }
+
+          @Override
+          public StringStringMapProto next() {
+            String key = iterator.next();
+            return StringStringMapProto.newBuilder()
+                .setValue(schedulingProperties.get(key)).setKey(key).build();
+          }
+
+          @Override
+          public void remove() {
+            throw new UnsupportedOperationException();
+          }
+        };
+      }
+    };
+    this.builder.addAllApplicationSchedulingProperties(values);
+  }
+
+  private void initApplicationSchedulingProperties() {
+    if (this.schedulingProperties != null) {
+      return;
+    }
+    ApplicationSubmissionContextProtoOrBuilder p = viaProto ? proto : builder;
+    List<StringStringMapProto> properties = p
+        .getApplicationSchedulingPropertiesList();
+    this.schedulingProperties = new HashMap<String, String>(properties.size());
+    for (StringStringMapProto envProto : properties) {
+      this.schedulingProperties.put(envProto.getKey(), envProto.getValue());
+    }
+  }
+
+  @Override
+  public synchronized
+      Map<String, String> getApplicationSchedulingPropertiesMap() {
+    initApplicationSchedulingProperties();
+    return this.schedulingProperties;
+  }
+
+  @Override
+  public synchronized void setApplicationSchedulingPropertiesMap(
+      Map<String, String> schedulingPropertyMap) {
+    if (schedulingPropertyMap == null) {
+      return;
+    }
+    initApplicationSchedulingProperties();
+    this.schedulingProperties.clear();
+    this.schedulingProperties.putAll(schedulingPropertyMap);
+  }
+}
