@@ -18,9 +18,14 @@
 
 package org.apache.hadoop.yarn.event;
 
-import com.google.common.annotations.VisibleForTesting;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.yarn.metrics.EventTypeMetrics;
+import org.apache.hadoop.yarn.util.Clock;
+import org.apache.hadoop.yarn.util.MonotonicClock;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import org.slf4j.Marker;
+import org.slf4j.MarkerFactory;
 import org.apache.hadoop.service.AbstractService;
 import org.apache.hadoop.util.ShutdownHookManager;
 import org.apache.hadoop.yarn.exceptions.YarnRuntimeException;
@@ -45,8 +50,14 @@ public class EventDispatcher<T extends Event> extends
   private final Thread eventProcessor;
   private volatile boolean stopped = false;
   private boolean shouldExitOnError = true;
+  private EventTypeMetrics metrics;
 
-  private static final Log LOG = LogFactory.getLog(EventDispatcher.class);
+  private static final Logger LOG =
+      LoggerFactory.getLogger(EventDispatcher.class);
+  private static final Marker FATAL =
+      MarkerFactory.getMarker("FATAL");
+
+  private Clock clock = new MonotonicClock();
 
   private final class EventProcessor implements Runnable {
     @Override
@@ -63,7 +74,14 @@ public class EventDispatcher<T extends Event> extends
         }
 
         try {
-          handler.handle(event);
+          if (metrics != null) {
+            long startTime = clock.getTime();
+            handler.handle(event);
+            metrics.increment(event.getType(),
+                clock.getTime() - startTime);
+          } else {
+            handler.handle(event);
+          }
         } catch (Throwable t) {
           // An error occurred, but we are shutting down anyway.
           // If it was an InterruptedException, the very act of
@@ -72,7 +90,7 @@ public class EventDispatcher<T extends Event> extends
             LOG.warn("Exception during shutdown: ", t);
             break;
           }
-          LOG.fatal("Error in handling event type " + event.getType()
+          LOG.error(FATAL, "Error in handling event type " + event.getType()
               + " to the Event Dispatcher", t);
           if (shouldExitOnError
               && !ShutdownHookManager.get().isShutdownInProgress()) {
@@ -131,6 +149,7 @@ public class EventDispatcher<T extends Event> extends
   public void disableExitOnError() {
     shouldExitOnError = false;
   }
+
   protected long getEventProcessorId() {
     return this.eventProcessor.getId();
   }
@@ -138,4 +157,9 @@ public class EventDispatcher<T extends Event> extends
   protected boolean isStopped() {
     return this.stopped;
   }
+
+  public void setMetrics(EventTypeMetrics metrics) {
+    this.metrics = metrics;
+  }
+
 }
