@@ -47,7 +47,7 @@ import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
-import org.mortbay.log.Log;
+import org.eclipse.jetty.util.log.Log;
 import org.mockito.Mockito;
 
 /** Test class to verify RollingLevelDBTimelineStore. */
@@ -374,7 +374,7 @@ public class TestRollingLevelDBTimelineStore extends TimelineStoreTestUtils {
     long start = System.currentTimeMillis();
     int num = 1000000;
 
-    Log.info("Start test for " + num);
+    Log.getLog().info("Start test for " + num);
 
     final String tezTaskAttemptId = "TEZ_TA";
     final String tezEntityId = "attempt_1429158534256_0001_1_00_000000_";
@@ -420,7 +420,7 @@ public class TestRollingLevelDBTimelineStore extends TimelineStoreTestUtils {
     }
 
     long duration = System.currentTimeMillis() - start;
-    Log.info("Duration for " + num + ": " + duration);
+    Log.getLog().info("Duration for " + num + ": " + duration);
   }
 
   @Test
