@@ -311,7 +311,7 @@ public class TestGpuDiscoverer {
   }
 
   @Test
-  public void testGpuDiscover() throws YarnException, IOException {
+  public void testGpuDiscover() throws YarnException {
     // Since this is more of a performance unit test, only run if
     // RunUserLimitThroughput is set (-DRunUserLimitThroughput=true)
     Assume.assumeTrue(
@@ -506,7 +506,7 @@ public class TestGpuDiscoverer {
     Configuration conf = new Configuration();
     conf.set(YarnConfiguration.NM_GPU_ALLOWED_DEVICES, "0:1,2:3");
 
-    GpuDiscoverer gpuSpy = spy(new GpuDiscoverer());
+    GpuDiscoverer gpuSpy = spy(GpuDiscoverer.class);
 
     gpuSpy.initialize(conf, binaryHelper);
     gpuSpy.getGpusUsableByYarn();
