@@ -57,6 +57,7 @@ public class RMAuditLogger {
     public static final String GET_APP_PRIORITY = "Get Application Priority";
     public static final String GET_APP_QUEUE = "Get Application Queue";
     public static final String GET_APP_ATTEMPTS = "Get Application Attempts";
+    public static final String GET_APP_REPORT = "Get Application Report";
     public static final String GET_APP_ATTEMPT_REPORT
         = "Get Application Attempt Report";
     public static final String GET_CONTAINERS = "Get Containers";
@@ -69,7 +70,6 @@ public class RMAuditLogger {
     public static final String FINISH_FAILED_APP = "Application Finished - Failed";
     public static final String FINISH_KILLED_APP = "Application Finished - Killed";
     public static final String REGISTER_AM = "Register App Master";
-    public static final String AM_ALLOCATE = "App Master Heartbeats";
     public static final String UNREGISTER_AM = "Unregister App Master";
     public static final String ALLOC_CONTAINER = "AM Allocated Container";
     public static final String RELEASE_CONTAINER = "AM Released Container";
@@ -78,16 +78,12 @@ public class RMAuditLogger {
     public static final String UPDATE_APP_TIMEOUTS =
         "Update Application Timeouts";
     public static final String GET_APP_TIMEOUTS = "Get Application Timeouts";
-    public static final String CHANGE_CONTAINER_RESOURCE =
-        "AM Changed Container Resource";
     public static final String SIGNAL_CONTAINER = "Signal Container Request";
 
     // Some commonly used descriptions
     public static final String UNAUTHORIZED_USER = "Unauthorized user";
     
     // For Reservation system
-    public static final String CREATE_NEW_RESERVATION_REQUEST = "Create " +
-        "Reservation Request";
     public static final String SUBMIT_RESERVATION_REQUEST = "Submit Reservation Request";
     public static final String UPDATE_RESERVATION_REQUEST = "Update Reservation Request";
     public static final String DELETE_RESERVATION_REQUEST = "Delete Reservation Request";
@@ -493,6 +489,7 @@ public class RMAuditLogger {
    * @param description Some additional information as to why the operation
    *                    failed.
    * @param appId ApplicationId in which operation was performed.
+   * @param attemptId Application Attempt Id in which operation was performed.
    *
    * <br><br>
    * Note that the {@link RMAuditLogger} uses tabs ('\t') as a key-val delimiter
