@@ -241,7 +241,15 @@ public class RMServerUtils {
 
   /**
    * Utility method to validate a list resource requests, by ensuring that the
-   * requested memory/vcore is non-negative and not greater than max
+   * requested memory/vcore is non-negative and not greater than max.
+   *
+   * @param ask resource request.
+   * @param maximumAllocation Maximum Allocation.
+   * @param queueName queue name.
+   * @param scheduler YarnScheduler.
+   * @param rmContext RMContext.
+   * @param nodeLabelsEnabled the node labels feature enabled.
+   * @throws InvalidResourceRequestException when there is invalid request.
    */
   public static void normalizeAndValidateRequests(List<ResourceRequest> ask,
       Resource maximumAllocation, String queueName, YarnScheduler scheduler,
@@ -264,9 +272,13 @@ public class RMServerUtils {
 
   /**
    * Validate increase/decrease request.
+   *
    * <pre>
    * - Throw exception when any other error happens
    * </pre>
+   * @param request SchedContainerChangeRequest.
+   * @param increase true, add container; false, decrease container.
+   * @throws InvalidResourceRequestException when there is invalid request.
    */
   public static void checkSchedContainerChangeRequest(
       SchedContainerChangeRequest request, boolean increase)
@@ -362,6 +374,7 @@ public class RMServerUtils {
    *                             application master.
    * @param appAttemptId         Application attempt Id
    * @throws InvalidContainerReleaseException
+   * an Application Master tries to release containers not belonging to it using.
    */
   public static void
       validateContainerReleaseRequest(List<ContainerId> containerReleaseList,
@@ -394,7 +407,7 @@ public class RMServerUtils {
    * @param module     like AdminService or NodeLabelManager
    * @param LOG        the logger to use
    * @return {@link UserGroupInformation} of the current user
-   * @throws IOException
+   * @throws IOException an I/O exception has occurred.
    */
   public static UserGroupInformation verifyAdminAccess(
       YarnAuthorizationProvider authorizer, String method, String module,
@@ -453,6 +466,17 @@ public class RMServerUtils {
     }
   }
 
+  public static YarnApplicationAttemptState convertRmAppAttemptStateToYarnApplicationAttemptState(
+      RMAppAttemptState currentState,
+      RMAppAttemptState previousState
+  ) {
+    return createApplicationAttemptState(
+        currentState == RMAppAttemptState.FINAL_SAVING
+        ? previousState
+        : currentState
+    );
+  }
+
   public static YarnApplicationAttemptState createApplicationAttemptState(
       RMAppAttemptState rmAppAttemptState) {
     switch (rmAppAttemptState) {
@@ -498,7 +522,9 @@ public class RMServerUtils {
   /**
    * Find all configs whose name starts with
    * YarnConfiguration.RM_PROXY_USER_PREFIX, and add a record for each one by
-   * replacing the prefix with ProxyUsers.CONF_HADOOP_PROXYUSER
+   * replacing the prefix with ProxyUsers.CONF_HADOOP_PROXYUSER.
+   *
+   * @param conf Configuration.
    */
   public static void processRMProxyUsersConf(Configuration conf) {
     Map<String, String> rmProxyUsers = new HashMap<String, String>();
