@@ -18,10 +18,10 @@
 
 package org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair;
 
-import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Preconditions;
-import com.google.common.collect.Lists;
-import com.google.common.util.concurrent.SettableFuture;
+import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
+import org.apache.hadoop.thirdparty.com.google.common.base.Preconditions;
+import org.apache.hadoop.thirdparty.com.google.common.collect.Lists;
+import org.apache.hadoop.thirdparty.com.google.common.util.concurrent.SettableFuture;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.apache.hadoop.classification.InterfaceAudience.LimitedPrivate;
@@ -463,6 +463,12 @@ public class FairScheduler extends
    * Add a new application to the scheduler, with a given id, queue name, and
    * user. This will accept a new app even if the user or queue is above
    * configured limits, but the app will not be marked as runnable.
+   *
+   * @param applicationId  applicationId.
+   * @param queueName queue name.
+   * @param user submit application user.
+   * @param isAppRecovering true, app recover; false, app not recover.
+   * @param placementContext application placement context.
    */
   protected void addApplication(ApplicationId applicationId,
       String queueName, String user, boolean isAppRecovering,
@@ -582,6 +588,10 @@ public class FairScheduler extends
 
   /**
    * Add a new application attempt to the scheduler.
+   *
+   * @param applicationAttemptId application AttemptId.
+   * @param transferStateFromPreviousAttempt transferStateFromPreviousAttempt.
+   * @param isAttemptRecovering true, attempt recovering;false, attempt not recovering.
    */
   protected void addApplicationAttempt(
       ApplicationAttemptId applicationAttemptId,
