@@ -113,5 +113,9 @@ public abstract class ACLsTestBase {
     return userClient;
   }
 
+  public Configuration getConf() {
+    return conf;
+  }
+
   protected abstract Configuration createConfiguration() throws IOException;
 }
