@@ -19,6 +19,7 @@
 package org.apache.hadoop.yarn.server.resourcemanager.monitor.capacity;
 
 import org.apache.hadoop.yarn.api.records.ApplicationAttemptId;
+import org.apache.hadoop.yarn.server.resourcemanager.monitor.capacity.mockframework.ProportionalCapacityPreemptionPolicyMockFramework;
 import org.apache.hadoop.yarn.server.resourcemanager.rmcontainer.RMContainer;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.CapacitySchedulerConfiguration;
 import org.apache.log4j.Level;
@@ -57,10 +58,10 @@ public class TestProportionalCapacityPreemptionPolicyPreemptToBalance
     policy.editSchedule();
 
     // I_A: A:30 B:35 C:35, preempt 5 from B and 15 from C to A
-    verify(mDisp, times(5)).handle(argThat(
+    verify(eventHandler, times(5)).handle(argThat(
         new TestProportionalCapacityPreemptionPolicy.IsPreemptionRequestFor(
             getAppAttemptId(2))));
-    verify(mDisp, times(15)).handle(argThat(
+    verify(eventHandler, times(15)).handle(argThat(
         new TestProportionalCapacityPreemptionPolicy.IsPreemptionRequestFor(
             getAppAttemptId(3))));
 
@@ -99,10 +100,10 @@ public class TestProportionalCapacityPreemptionPolicyPreemptToBalance
     policy.editSchedule();
 
     // I_A: A:33 B:33 C:33, preempt 7 from B and 17 from C to A
-    verify(mDisp, times(7)).handle(argThat(
+    verify(eventHandler, times(7)).handle(argThat(
         new TestProportionalCapacityPreemptionPolicy.IsPreemptionRequestFor(
             getAppAttemptId(2))));
-    verify(mDisp, times(17)).handle(argThat(
+    verify(eventHandler, times(17)).handle(argThat(
         new TestProportionalCapacityPreemptionPolicy.IsPreemptionRequestFor(
             getAppAttemptId(3))));
 
@@ -142,7 +143,7 @@ public class TestProportionalCapacityPreemptionPolicyPreemptToBalance
     policy.editSchedule();
 
     // I_A: A:15 B:42 C:43, preempt 7 from B and 17 from C to A
-    verify(mDisp, times(8)).handle(argThat(
+    verify(eventHandler, times(8)).handle(argThat(
         new TestProportionalCapacityPreemptionPolicy.IsPreemptionRequestFor(
             getAppAttemptId(3))));
 
@@ -178,7 +179,7 @@ public class TestProportionalCapacityPreemptionPolicyPreemptToBalance
     policy.editSchedule();
 
     // 21 containers will be preempted here
-    verify(mDisp, times(21)).handle(argThat(
+    verify(eventHandler, times(21)).handle(argThat(
         new TestProportionalCapacityPreemptionPolicy.
             IsPreemptionRequestFor(getAppAttemptId(2))));
 
@@ -240,7 +241,7 @@ public class TestProportionalCapacityPreemptionPolicyPreemptToBalance
     assertEquals(hasFifoSelector, true);
 
     // 21 containers will be preempted here
-    verify(mDisp, times(21)).handle(argThat(
+    verify(eventHandler, times(21)).handle(argThat(
         new TestProportionalCapacityPreemptionPolicy.
             IsPreemptionRequestFor(getAppAttemptId(2))));
 
